@@ -8,7 +8,9 @@ from typing import Any, Callable
 import click
 import coloredlogs  # type: ignore
 
-from . import __version__, core, sources, utils  # noqa: F401
+from . import __version__, core, iso639, utils
+from .sources.base import Parser
+
 from .exceptions import MangaNotFoundError
 
 click.option: Callable[..., Any] = functools.partial(click.option, show_default=True)  # type: ignore
@@ -19,29 +21,8 @@ VERBOSITY = [
     for level in ("CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG")
 ]
 
-CONFIG = utils.Config()
-LANG = CONFIG["lang"]
-
-
-@click.group()
-@click.version_option(__version__)
-@click.option(
-    "-v", "--verbose", "verbosity", help="be more chatty", default=4, count=True
-)
-def cli(verbosity):
-    """Manga browser/downloader.
-
-    Once a manga has been added to tankobon using 'tankobon add <url>',
-    you can refer to it using its shorthash (first 8 characters of its SHA512 hash).
-
-    Executing 'tankobon list' will show the shorthashes of all manga added to tankobon.
-    """
-    # set up logger
-    coloredlogs.install(
-        level=VERBOSITY[verbosity - 1],
-        fmt=" %(levelname)-8s :: %(message)s",
-        logger=_log,
-    )
+CONFIG = utils.CONFIG
+CONFIG_TYPES = [int, float, str]
 
 
 def prettyprint(dict_):
@@ -67,6 +48,70 @@ def _load(shorthash, cache):
         raise click.Abort()
 
 
+@click.group()
+@click.version_option(__version__)
+@click.option(
+    "-v", "--verbose", "verbosity", help="be more chatty", default=4, count=True
+)
+def cli(verbosity):
+    """Manga browser/downloader.
+
+    Once a manga has been added to tankobon using 'tankobon add <url>',
+    you can refer to it using its shorthash (first 8 characters of its SHA512 hash).
+
+    Executing 'tankobon list' will show the shorthashes of all manga added to tankobon.
+    """
+    # set up logger
+    coloredlogs.install(
+        level=VERBOSITY[verbosity - 1],
+        fmt=" %(levelname)-8s :: %(message)s",
+        logger=_log,
+    )
+
+
+def to_native(s):
+    if s in ("true", "false"):
+        return s == "true"
+
+    for _type in CONFIG_TYPES:
+        try:
+            return _type(s)
+        except ValueError:
+            pass
+
+    return None
+
+
+@cli.command()
+@click.argument("pair", nargs=-1)
+def config(pair):
+    """Configure tankobon.
+
+    \b
+    This command can be used several ways:
+
+    \b
+    'tankobon config' - list all keys and values
+    'tankobon config (key)' - show value for key
+    'tankobon config (key) (value)' - set key to value
+    """
+
+    pair = list(pair[:2])
+    while len(pair) < 2:
+        pair.append(None)
+
+    key, value = pair
+
+    if key is None:
+        for key, value in CONFIG.items():
+            click.echo(f"{key}: {value}")
+    else:
+        if value is None:
+            click.echo(CONFIG[key])
+        else:
+            CONFIG[key] = to_native(value)
+
+
 @cli.command()
 @click.argument("shorthash")
 @click.option("-c", "--chapter", help="show info only for a specific chapter")
@@ -76,21 +121,28 @@ def info(shorthash, chapter):
     with core.Cache() as cache:
 
         manga = _load(shorthash, cache)
+        info = manga.info
 
         if chapter:
-            prettyprint(manga.chapters[chapter].__dict__)
+            prettyprint(manga.chapters[chapter][CONFIG["lang"]].__dict__)
         else:
             prettyprint(manga.meta.__dict__)
 
-            click.echo(manga.summary())
+            prettyprint(
+                {
+                    "languages": [
+                        f"{iso639.DATASET[lang].native_name} ({lang})"
+                        for lang in info["langs"]
+                    ]
+                }
+            )
 
-            info = manga.info
+            click.echo(manga.summary(lang=CONFIG["lang"], link=False))
 
             click.echo(
                 "summary: "
                 f"{utils.plural(len(info['volumes']), 'volume')}, "
-                f"{utils.plural(info['chapters'], 'chapter')}\n"
-                f"languages: {', '.join(info['langs'])}"
+                f"{utils.plural(info['chapters'], 'chapter')}"
             )
 
 
@@ -99,7 +151,7 @@ def _list():
     """List all manga in the cache."""
 
     prettyprint(
-        {"supported websites": [cls.domain.pattern for cls in core.Parser.registered]}
+        {"supported websites": [cls.domain.pattern for cls in Parser.registered]}
     )
 
     with core.Cache() as cache:
@@ -115,7 +167,7 @@ def _list():
 
 
 def _refresh(manga):
-    parser = core.Parser.parser(manga.meta.url)
+    parser = Parser.by_url(manga.meta.url)
     parser.add_chapters(manga)
 
 
@@ -134,7 +186,7 @@ def add(url):
             )
             return
 
-        parser = core.Parser.parser(url)
+        parser = Parser.by_url(url)
         manga = parser.create(url)
 
         _refresh(manga)
@@ -190,10 +242,10 @@ def download(shorthash, path, cids, force):
     """Download a manga by shorthash."""
 
     cache = core.Cache()
-    parser = core.Parser.parser(shorthash)
     downloader = core.Downloader(path)
 
     manga = _load(shorthash, cache)
+    parser = Parser.by_url(manga.meta.url)
 
     if cids is None:
         if not click.confirm(
@@ -204,12 +256,12 @@ def download(shorthash, path, cids, force):
         chapters = []
 
         for _, langs in manga.chapters.items():
-            chapter = langs.get(LANG)
+            chapter = langs.get(CONFIG["lang"])
             if chapter is not None:
                 chapters.append(chapter)
 
     else:
-        chapters = manga.select(cids, lang=LANG)
+        chapters = manga.select(cids, lang=CONFIG["lang"])
 
     for chapter in chapters:
         click.echo(f"downloading chapter {chapter.id}")
@@ -249,7 +301,7 @@ def pdfify(path, chapters, output):
         else:
             chapters = chapters.split(",")
 
-        downloader.pdfify(chapters, output, lang=LANG)
+        downloader.pdfify(chapters, output, lang=CONFIG["lang"])
 
 
 @cli.command("gui")
@@ -267,3 +319,10 @@ def _gui():
 
     else:
         gui.main()
+
+
+def main():
+    try:
+        cli()
+    finally:
+        CONFIG.close()
