@@ -1,25 +1,20 @@
 # coding: utf8
 """Core functionality of tankobon."""
 
-from __future__ import annotations
-
-import abc
 import concurrent.futures as cfutures
 import gzip
 import logging
 import pathlib
-import re
 import shutil
-from typing import cast, Callable, Dict, List, Optional, Type, Union
+from typing import cast, Callable, Dict, List, Optional, Union
 
-import bs4  # type: ignore
 import fpdf  # type: ignore
 import imagesize  # type: ignore
 import natsort  # type: ignore
 import requests.exceptions
 
 from . import models, utils
-from .exceptions import MangaNotFoundError, PagesNotFoundError, UnknownDomainError
+from .exceptions import MangaNotFoundError, PagesNotFoundError
 
 _log = logging.getLogger("tankobon")
 
@@ -29,119 +24,6 @@ A4_HEIGHT = 297
 SHORT_HASH_LEN = 8
 
 
-class Parser(abc.ABC):
-
-    # This allows subclasses to be registered.
-    registered: List[Type[Parser]] = []
-
-    def __init__(self):
-        self.session = utils.UserSession()
-
-    def create(self, url: str) -> models.Manga:
-        """Create a new manga.
-
-        Args:
-            url: The manga url.
-
-        Returns:
-            A Manga object.
-        """
-
-        metadata = self.metadata(url)
-        return models.Manga(metadata)
-
-    @classmethod
-    def parser(cls, url: str) -> Parser:
-        """Get the appropiate parser subclass for the domain in url.
-
-        Args:
-            url: The url to get the subclass for.
-
-        Returns:
-            The subclass instance that can be used to parse the url.
-
-        Raises:
-            UnknownDomainError, if there is no registered subclass for the url domain.
-        """
-
-        for subclass in cls.registered:
-            if subclass.domain.search(url):  # type: ignore
-                return subclass()
-
-        raise UnknownDomainError(f"no source found for url '{url}'")
-
-    @property
-    @abc.abstractmethod
-    def domain(self) -> str:
-        pass
-
-    @abc.abstractmethod
-    def metadata(self, url: str) -> models.Metadata:
-        """Parse metadata for a manga url.
-
-        Args:
-            url: The manga url.
-
-        Returns:
-            The Metadata object.
-        """
-
-    @abc.abstractmethod
-    def add_chapters(self, manga: models.Manga):
-        """Add chapters to the manga.
-
-        This method should add every chapter in the manga as a Chapter object:
-
-        ```python
-        def chapters(self, manga):
-            for ... in ...:
-                # do your parsing here
-                manga.add(Chapter(...))
-        ```
-
-        Only the 'url' and 'id' args are required when creating a Chapter.
-        The other fields are optional and have default values (see `help(tankobon.models.Chapter)`).
-
-        Args:
-            manga: The manga object.
-        """
-
-    @abc.abstractmethod
-    def add_pages(self, chapter: models.Chapter):
-        """Add pages to the chapter in the manga as a list of urls.
-        The pages must be in ascending order.
-
-        This method should assign pages to the chapter:
-
-        ```python
-        def pages(self, chapter):
-            # do your parsing here
-            chapter.pages = [...]  # assign directly to the chapter's pages.
-        ```
-
-        Args:
-            chapter: The chapter object (already added to the manga).
-        """
-
-    def soup(self, url: str) -> bs4.BeautifulSoup:
-        """Get a soup from a url.
-
-        Args:
-            url: The url to get a soup from.
-
-        Returns:
-            The soup of the url.
-        """
-
-        return utils.soup(url, session=self.session)
-
-    def __init_subclass__(cls, **kwargs):
-        super().__init_subclass__(**kwargs)
-
-        cls.domain = re.compile(cls.domain)
-        cls.registered.append(cls)
-
-
 class Cache(utils.PersistentDict):
     """A manga cache.
 
@@ -264,6 +146,7 @@ class Downloader:
 
         self.path = path
 
+        self.config = utils.CONFIG
         self.session = utils.UserSession()
         self.manifest = utils.PersistentDict(self.path / self.MANIFEST)
 
@@ -271,6 +154,15 @@ class Downloader:
         self.session.close()
         self.manifest.close()
 
+    def downloaded(self, chapter: models.Chapter) -> bool:
+        """Check whether a chapter has been downloaded or not."""
+        try:
+            self.manifest[chapter.id][chapter.lang]
+        except KeyError:
+            return False
+        else:
+            return True
+
     def download(
         self,
         chapter: models.Chapter,
@@ -310,7 +202,9 @@ class Downloader:
         pages = []
         total = len(chapter.pages)
 
-        with cfutures.ThreadPoolExecutor(max_workers=8) as pool:
+        with cfutures.ThreadPoolExecutor(
+            max_workers=self.config["download.rate_limit"]
+        ) as pool:
             futures = {
                 pool.submit(self.session.get, url): count
                 for count, url in enumerate(chapter.pages)
