@@ -13,6 +13,7 @@ sources:
 {supported}
 """
 
+import functools
 import pathlib
 import signal
 import sys
@@ -23,6 +24,8 @@ from PySide6.QtCore import Qt, Signal, QSize
 from PySide6.QtGui import QAction, QIcon, QPixmap
 from PySide6.QtWidgets import (
     QApplication,
+    QComboBox,
+    QDialog,
     QDialogButtonBox,
     QGridLayout,
     QHBoxLayout,
@@ -36,8 +39,11 @@ from PySide6.QtWidgets import (
     QProgressDialog,
     QScrollArea,
     QSizePolicy,
+    QSpinBox,
     QSplashScreen,
-    QTextBrowser,
+    QTabWidget,
+    QTableWidget,
+    # QTableWidgetItem,
     QToolBar,
     QToolButton,
     QVBoxLayout,
@@ -45,26 +51,27 @@ from PySide6.QtWidgets import (
     QWidgetItem,
 )
 
-from .. import core, models, sources  # noqa: F401
-from ..utils import Config
-from . import resources, template, utils  # noqa: F401
+import natsort  # type: ignore
+
+from .. import core, iso639, models
+from ..sources.base import Parser
+from ..utils import CONFIG
 
 from ..__version__ import __version__
 
+from . import resources, template, utils  # noqa: F401
+
 _app = QApplication([])
+_app.setAttribute(Qt.AA_UseHighDpiPixmaps)
+
 QStyle = _app.style()
 
 LOGO = QPixmap(":/logo.jpg")
 
 HOME = pathlib.Path.home()
 
-CONFIG = Config()
-LANG = CONFIG["lang"]
-
 CACHE = core.Cache()
 
-MAX_COL = 5
-
 T_ADD = "Add Manga"
 T_DELETE = "Delete Manga"
 T_DOWNLOAD = "Download Manga"
@@ -156,34 +163,6 @@ class MessageBox(QMessageBox):
         return msgbox.exec()
 
 
-class AboutBox(MessageBox):
-    def __init__(self, *args):
-        super().__init__(*args)
-
-        self.setWindowTitle("About")
-
-        # build table of supported sources
-        supported = []
-        for cls in core.Parser.registered:
-            supported.append(f"`{cls.__module__}` ({cls.domain.pattern})  ")
-
-        self.setTextFormat(Qt.MarkdownText)
-        self.setText(
-            __doc__.format(
-                version=__version__,
-                supported="\n".join(supported),
-            )
-        )
-
-        self.setAttribute(Qt.WA_DeleteOnClose)
-
-        small_logo = LOGO.scaled(
-            QSize(256, 256), Qt.KeepAspectRatio, Qt.SmoothTransformation
-        )
-
-        self.setIconPixmap(small_logo)
-
-
 def _excepthook(ex_type, ex_value, ex_traceback):
     MessageBox.crit(
         "An exception occured.",
@@ -226,6 +205,112 @@ class ProgressDialog(QProgressDialog):
         self.setAttribute(Qt.WA_DeleteOnClose, True)
 
 
+class LanguageComboBox(QComboBox):
+    def __init__(self):
+        super().__init__()
+
+        index = None
+
+        for code, lang in iso639.DATASET.items():
+            self.addItem(f"{lang.native_name} ({code})", code)
+            if code == CONFIG["lang"]:
+                index = self.count() - 1
+
+        self.setCurrentIndex(index)
+
+        self.currentIndexChanged.connect(self.onCurrentIndexChanged)
+
+    def onCurrentIndexChanged(self, index):
+        CONFIG["lang"] = self.currentData()
+
+
+class RateLimitSpinBox(QSpinBox):
+    def __init__(self):
+        super().__init__()
+        self.setMinimum(1)
+        self.setValue(CONFIG["download.rate_limit"])
+        self.valueChanged.connect(self.onValueChanged)
+
+    def onValueChanged(self, value):
+        CONFIG["download.rate_limit"] = value
+
+
+class SettingsTab(QWidget):
+    def __init__(self):
+        super().__init__()
+        self.layout = QVBoxLayout(self)
+
+
+class Settings(QDialog):
+    def __init__(self, parent):
+        super().__init__(parent)
+        self.setWindowTitle("Settings")
+
+        layout = QVBoxLayout(self)
+
+        tabs = QTabWidget()
+        tabs.addTab(self.general(), "General")
+        tabs.addTab(self.downloads(), "Downloads")
+        layout.addWidget(tabs)
+
+        buttons = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        buttons.accepted.connect(self.accept)
+        buttons.rejected.connect(self.reject)
+        layout.addWidget(buttons)
+
+    def general(self):
+        tab = SettingsTab()
+
+        tab.layout.addWidget(QLabel("Language"))
+        tab.layout.addWidget(LanguageComboBox())
+
+        return tab
+
+    def downloads(self):
+        tab = SettingsTab()
+
+        tab.layout.addWidget(QLabel("Rate Limit"))
+
+        spinbox = RateLimitSpinBox()
+        spinbox.setToolTip(
+            (
+                "The maximum number of requests that can be made concurrently.\n"
+                "Lower values reduce bandwidth usage but downloads will be slower."
+            )
+        )
+        tab.layout.addWidget(spinbox)
+
+        return tab
+
+
+class AboutBox(MessageBox):
+    def __init__(self, *args):
+        super().__init__(*args)
+
+        self.setWindowTitle("About")
+
+        # build table of supported sources
+        supported = []
+        for cls in Parser.registered:
+            supported.append(f"`{cls.__module__}` ({cls.domain.pattern})  ")
+
+        self.setTextFormat(Qt.MarkdownText)
+        self.setText(
+            __doc__.format(
+                version=__version__,
+                supported="\n".join(supported),
+            )
+        )
+
+        self.setAttribute(Qt.WA_DeleteOnClose)
+
+        small_logo = LOGO.scaled(
+            QSize(256, 256), Qt.KeepAspectRatio, Qt.SmoothTransformation
+        )
+
+        self.setIconPixmap(small_logo)
+
+
 # A manga item.
 class Item(QListWidgetItem):
     def __init__(self, meta: models.Metadata):
@@ -250,27 +335,29 @@ class ItemInfoBox(QWidget):
         title = TitleLabel(f"<h2><i>{meta.title}</i></h2>")
         layout.addWidget(title, 0, 0, *SPAN)
 
-        cover = QPixmap()
+        self.cover = QPixmap()
+
         try:
             manga_path = CACHE.root / meta.hash
             cover_path = next(manga_path.glob("cover.*"))
 
         except StopIteration:
-            cover = QPixmap(":/missing.jpg")
+            self.cover.load(":/missing.jpg")
 
         else:
-            cover.load(str(cover_path))
+            self.cover.load(str(cover_path))
 
-        # scale cover
-        self.cover = cover.scaled(
-            self.width(),
+        self.cover = self.cover.scaled(
+            int(self.width() / 2),
             self.height(),
             Qt.KeepAspectRatio,
             Qt.SmoothTransformation,
         )
+
         self.cover_label = QLabel()
+        self.cover_label.setScaledContents(True)
         self.cover_label.setPixmap(self.cover)
-        self.resizeCover()
+
         layout.addWidget(self.cover_label, 1, 0, *SPAN)
 
         if meta.alt_titles is not None:
@@ -319,20 +406,29 @@ class ItemInfoBox(QWidget):
         source.setWordWrap(True)
         layout.addWidget(source, 6, 1)
 
-    def resizeCover(self):
-        self.cover = self.cover.scaled(
-            int(self.width() / 2),
-            self.height(),
-            Qt.KeepAspectRatio,
-            Qt.SmoothTransformation,
+        langs_header = SubtitleLabel("Languages")
+        layout.addWidget(langs_header, 7, 0)
+
+        manga = CACHE[meta.hash]
+        langs_set = set()
+
+        for chapter in manga["chapters"].values():
+            langs_set.update(chapter.keys())
+
+        langs = QLabel(
+            "<br>".join(
+                f"{iso639.DATASET[lang].native_name} ({lang})" for lang in langs_set
+            )
         )
-        self.cover_label.setPixmap(self.cover)
+        layout.addWidget(langs, 7, 1)
 
 
 # A list of manga items in the sidebar.
 class ItemList(QListWidget):
     def __init__(self):
         super().__init__()
+        self.setSortingEnabled(True)
+
         self.hashs = set()
 
         for _, manga in CACHE.data.items():
@@ -351,6 +447,138 @@ class ItemList(QListWidget):
 MANGA_ITEMS = ItemList()
 
 
+class PageViewToolBar(QToolBar):
+
+    BUTTONS = [
+        ("start", "chevrons-left"),
+        ("previous", "chevron-left"),
+        ("pageno", ""),
+        ("next", "chevron-right"),
+        ("end", "chevrons-right"),
+    ]
+
+    setPage = Signal(int)
+
+    def __init__(self, total):
+
+        super().__init__()
+        self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Maximum)
+
+        self.pageno = 1
+        self.total = total
+        self.label = QLabel()
+
+        self.setPage.connect(self.onSetPage)
+
+        for method_name, icon_path in self.BUTTONS:
+
+            if method_name == "pageno":
+                self.addWidget(self.label)
+                continue
+
+            method = getattr(self, method_name)
+            tooltip = f"{method_name.title()} page..."
+
+            icon = utils.icon(icon_path)
+
+            action = self.addAction(icon, tooltip)
+            action.triggered.connect(method)
+
+    def onSetPage(self, pageno):
+        self.pageno = pageno
+        self.label.setText(f"{pageno} / {self.total}")
+
+        actions = self.actions()
+        for action in actions:
+            action.setEnabled(True)
+
+        # disable start/prev or end/next actions on the first and last page respectively.
+        disable = []
+
+        if self.pageno == 1:
+            disable = actions[:2]
+        elif self.pageno == self.total:
+            disable = actions[2:]
+
+        for action in disable:
+            action.setEnabled(False)
+
+    def start(self):
+        self.setPage.emit(1)
+
+    def previous(self):
+        self.setPage.emit(self.pageno - 1)
+
+    def next(self):
+        self.setPage.emit(self.pageno + 1)
+
+    def end(self):
+        self.setPage.emit(self.total)
+
+
+class PageView(QWidget):
+    def __init__(self, parent, pages):
+        super().__init__(parent, Qt.Window)
+        self.layout = QVBoxLayout(self)
+
+        self.pages = pages
+
+        self.label = QLabel()
+        self.label.setSizePolicy(
+            QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding
+        )
+        self.label.setScaledContents(True)
+
+        self.layout.addWidget(self.label, 0, Qt.AlignCenter)
+
+        self.toolbar = PageViewToolBar(len(self.pages))
+        self.toolbar.setPage.connect(self.onSetPage)
+
+        self.layout.addWidget(self.toolbar, 0, Qt.AlignCenter)
+
+        self._height = self.label.height() + self.toolbar.height() / 2
+
+        self.toolbar.start()
+
+    def onSetPage(self, pageno):
+        pixmap = QPixmap(self.pages[pageno - 1])
+
+        height = self._height
+        width = (pixmap.width() / pixmap.height()) * height
+
+        print(width, height)
+
+        self.label.resize(width, height)
+        self.label.setPixmap(pixmap)
+
+
+def _download(manga, chapters, downloader):
+    dialog = ProgressDialog()
+
+    parser = Parser.by_url(manga.meta.url)
+
+    for count, chapter in enumerate(chapters):
+
+        dialog.setLabelText(f"Downloading chapter {chapter.id}...")
+
+        if not chapter.pages:
+            parser.add_pages(chapter)
+
+        total = len(chapter.pages)
+
+        # HACK: make the progress bar display properly during first iteration
+        dialog.setMaximum(total)
+        dialog.setValue(total - 1)
+        dialog.setValue(0)
+
+        if dialog.wasCanceled():
+            break
+
+        downloader.download(chapter, progress=dialog.setValue)
+
+        dialog.setValue(total)
+
+
 # Toolbar at the bottom of the window.
 # This shows a bunch of buttons to manage manga items (add, remove, etc.)
 class ToolBar(QToolBar):
@@ -358,21 +586,17 @@ class ToolBar(QToolBar):
     deletedManga = Signal()
 
     BUTTONS = [
-        ("add", ":/plus.svg"),
+        ("add", "plus"),
         (
             "delete",
-            ":/minus.svg",
+            "minus",
         ),
         (
             "refresh",
-            ":/refresh-cw.svg",
+            "refresh-cw",
         ),
-        (
-            "download",
-            ":/download.svg",
-        ),
-        ("view", ":/eye.svg"),
-        ("locate", ":/folder.svg"),
+        # ("download", "download",),
+        ("locate", "folder"),
     ]
 
     def __init__(self):
@@ -384,21 +608,14 @@ class ToolBar(QToolBar):
 
         self.summaries = {}
 
-        bg_is_dark = utils.is_dark(_app.palette().window().color())
-
         for method_name, icon_path in self.BUTTONS:
 
             method = getattr(self, method_name)
             tooltip = f"{method_name.title()} a manga..."
 
-            if bg_is_dark:
-                icon = QIcon(icon_path.replace(".svg", "-light.svg"))
-            else:
-                icon = QIcon(icon_path)
-
             action = QAction()
             action.setToolTip(tooltip)
-            action.setIcon(icon)
+            action.setIcon(utils.icon(icon_path))
             action.triggered.connect(method)
 
             button = QToolButton()
@@ -443,7 +660,7 @@ class ToolBar(QToolBar):
     def _refresh(self, manga):
 
         with SpinningCursor():
-            parser = core.Parser.parser(manga.meta.url)
+            parser = Parser.by_url(manga.meta.url)
             parser.add_chapters(manga)
 
             CACHE.dump(manga)
@@ -473,7 +690,7 @@ class ToolBar(QToolBar):
             return
 
         try:
-            parser = core.Parser.parser(url)
+            parser = Parser.by_url(url)
         except core.UnknownDomainError:
             MessageBox.warn(
                 T_ADD,
@@ -511,33 +728,8 @@ class ToolBar(QToolBar):
 
         manga = _load_manga(self.selected.meta.hash)
         self._refresh(manga)
-
-    def _download(self, manga, chapters):
-        dialog = ProgressDialog(self)
-
-        parser = core.Parser.parser(manga.meta.url)
-
-        with core.Downloader(CACHE.root / manga.meta.hash) as downloader:
-            for count, chapter in enumerate(chapters):
-
-                dialog.setLabelText(f"Downloading chapter {chapter.id}...")
-
-                if not chapter.pages:
-                    parser.add_pages(chapter)
-
-                total = len(chapter.pages)
-
-                # HACK: make the progress bar display properly during first iteration
-                dialog.setMaximum(total)
-                dialog.setValue(total - 1)
-                dialog.setValue(0)
-
-                if dialog.wasCanceled():
-                    break
-
-                downloader.download(chapter, progress=dialog.setValue)
-
-                dialog.setValue(total)
+        # reload view
+        MANGA_ITEMS.itemClicked.emit(self.selected)
 
     def download(self):
         if not self.ensureSelected("download"):
@@ -556,17 +748,12 @@ class ToolBar(QToolBar):
         if dialog_code == QInputDialog.Rejected:
             return
 
-        chapters = manga.select(dialog.textValue(), lang=LANG)
+        chapters = manga.select(dialog.textValue(), lang=CONFIG["lang"])
         if not chapters:
             MessageBox.warn(T_DOWNLOAD, "Chapters/range is invalid.")
 
-        self._download(manga, chapters)
-
-    def view(self):
-        if not self.ensureSelected("view"):
-            return
-
-        MessageBox.info("oops", "Sorry! The viewer isn't implemented yet.")
+        with core.Downloader(CACHE.root / manga.meta.hash) as downloader:
+            _download(manga, chapters, downloader)
 
     def locate(self):
         if not self.ensureSelected("locate"):
@@ -584,41 +771,141 @@ class MenuBar(QMenuBar):
 
         file = self.addMenu("File")
 
+        file_settings = QAction("Settings", self)
+        file_settings.triggered.connect(self.settings)
+        file.addAction(file_settings)
+
         file_quit = QAction("Quit", self)
         file_quit.triggered.connect(_app.quit)
-
         file.addAction(file_quit)
 
         help = self.addMenu("Help")
 
         help_tankobon = QAction("About tankobon", self)
         help_tankobon.triggered.connect(self.about)
-
         help.addAction(help_tankobon)
 
         help_qt = QAction("About Qt", self)
         help_qt.triggered.connect(_app.aboutQt)
-
         help.addAction(help_qt)
 
+    def settings(self):
+        settings_dialog = Settings(self)
+        settings_dialog.exec()
+        self.parentWidget().reload()
+
     def about(self):
         about_box = AboutBox(self)
         about_box.exec()
 
 
+# A grid of manga chapters.
+class ChapterView(QTableWidget):
+    def __init__(self, manga):
+        super().__init__()
+        self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
+
+        self.setShowGrid(False)
+        self.setFocusPolicy(Qt.NoFocus)
+        self.verticalHeader().setVisible(False)
+        self.horizontalHeader().setStretchLastSection(True)
+
+        self.buttons = {}
+
+        self.manga = manga
+        self.downloader = core.Downloader(CACHE.root / self.manga.meta.hash)
+
+        chapters = natsort.natsorted(
+            [
+                chapter
+                for chapter, langs in manga.chapters.items()
+                if CONFIG["lang"] in langs
+            ]
+        )
+
+        self.setRowCount(len(chapters))
+        self.setColumnCount(4)
+
+        self.setHorizontalHeaderLabels(["status", "volume", "chapter", "title"])
+
+        for row, chapter in enumerate(chapters):
+            self.addChapter(row, manga.chapters[chapter][CONFIG["lang"]])
+
+    def addChapter(self, row, chapter):
+
+        if self.downloader.downloaded(chapter):
+            icon = utils.icon("eye")
+        else:
+            icon = utils.icon("download")
+
+        button = QToolButton()
+        button.setStyleSheet("border: none;")
+        button.setIcon(icon)
+        button.clicked.connect(functools.partial(self.onClicked, chapter))
+        self.buttons[chapter.id] = button
+        self.setCellWidget(row, 0, button)
+
+        volume = QLabel(chapter.volume or "(empty)")
+        volume.setAlignment(Qt.AlignCenter)
+        self.setCellWidget(row, 1, volume)
+
+        id = QLabel(chapter.id)
+        id.setAlignment(Qt.AlignCenter)
+        self.setCellWidget(row, 2, id)
+
+        title = QLabel(f'<a href="{chapter.url}">{chapter.title or "(empty)"}</a>')
+        title.setOpenExternalLinks(True)
+        self.setCellWidget(row, 3, title)
+
+    def onClicked(self, chapter):
+        if not self.downloader.downloaded(chapter):
+            _download(self.manga, [chapter], self.downloader)
+            self.downloader.manifest.sync()
+
+        self.buttons[chapter.id].setIcon(utils.icon("eye"))
+
+        # pages = self.downloader.manifest[chapter.id][chapter.lang]
+
+        # page_view = PageView(self, pages)
+        # page_view.showMaximized()
+
+        utils.xopen(str(self.downloader.path / chapter.id / chapter.lang))
+
+
+# The manga chapters plus description.
+class SummaryView(QWidget):
+    def __init__(self, manga):
+        super().__init__()
+
+        self.layout = QVBoxLayout(self)
+
+        chapters = ChapterView(manga)
+
+        self.layout.addWidget(chapters)
+
+        desc = QLabel()
+        desc.setWordWrap(True)
+        desc.setTextFormat(Qt.RichText)
+        desc.setOpenExternalLinks(True)
+        desc.setText(template.create(manga, lang=CONFIG["lang"]))
+
+        self.layout.addWidget(desc)
+
+
 # The combined manga item list plus preview.
 class View(QWidget):
     def __init__(self):
         super().__init__()
         self.layout = QHBoxLayout(self)
 
+        self.selected = None
         self.pixmap_cache = {}
 
         # The split view at first shows the list of manga items and the default item view.
         # After a manga has been selected, there will be a total of three widgets:
         # - Manga item list
-        # - Manga info.
-        # - Manga cover
+        # - Manga description
+        # - Manga info (incl. cover)
         self.layout.addWidget(MANGA_ITEMS)
         self.layout.addWidget(self.default())
 
@@ -636,24 +923,19 @@ class View(QWidget):
 
     def onSelectedManga(self, manga_item):
         self.deleteLast()
+        self.selected = manga_item
 
         manga = _load_manga(manga_item.meta.hash)
 
-        text = QTextBrowser()
-        text.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
-        text.setReadOnly(True)
-        text.setOpenExternalLinks(True)
-        text.document().setDefaultStyleSheet(
-            utils.resource(":/view.css").decode("utf8")
-        )
-        text.setHtml(template.create(manga))
+        summary = SummaryView(manga)
 
-        self.layout.addWidget(text)
+        self.layout.addWidget(summary)
 
         infobox = ItemInfoBox(manga_item)
         scroll = QScrollArea()
         scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        scroll.verticalScrollBar().setStyleSheet("height:0px;")
+        scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+        scroll.setFrameShape(scroll.NoFrame)
         scroll.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
         scroll.setWidget(infobox)
 
@@ -664,6 +946,14 @@ class View(QWidget):
 
         self.layout.addWidget(self.default())
 
+    def reload(self):
+        if self.selected is not None:
+            self.onSelectedManga(self.selected)
+
+        else:
+            # no manga selected yet.
+            self.onDeletedManga()
+
 
 # Main window.
 class Root(QMainWindow):
@@ -691,6 +981,9 @@ class Root(QMainWindow):
 
         self.setCentralWidget(wrapper)
 
+    def reload(self):
+        self.view.reload()
+
     def confirmQuit(self):
         reply = MessageBox.ask(
             "Quit?",
@@ -700,7 +993,7 @@ class Root(QMainWindow):
 
     def closeEvent(self, event):
         if self.confirmQuit():
-
+            CONFIG.close()
             CACHE.close()
             event.accept()
         else:
