@@ -5,7 +5,6 @@ import collections
 import gzip
 import json
 import logging
-import os
 import pathlib
 import re
 from typing import Optional, Union
@@ -158,10 +157,14 @@ class PersistentDict(collections.UserDict):
     # }
 
     # It can also be used without a context manager.
-    # Just remember to close() it, or any changes won't be written to disk!
+    # Just remember to sync() or close() it, or any changes won't be written to disk!
 
     d = PersistentDict(file)
     d["baz"] = 42
+    d.sync()
+
+    # other operations...
+
     d.close()
     ```
     """
@@ -190,7 +193,7 @@ class PersistentDict(collections.UserDict):
         except FileNotFoundError:
             pass
 
-    def close(self):
+    def sync(self):
         if self.compress:
             f = gzip.open(self.path, "wt")
         else:
@@ -199,6 +202,10 @@ class PersistentDict(collections.UserDict):
         json.dump(self.data, f, indent=2)
         f.close()
 
+    def close(self):
+        # Alias of self.sync, for now.
+        self.sync()
+
     def __enter__(self):
         return self
 
@@ -209,7 +216,7 @@ class PersistentDict(collections.UserDict):
 class Config(PersistentDict):
 
     CONFIG = "config.json"
-    DEFAULTS = {"lang": "en"}
+    DEFAULTS = {"lang": "en", "download.rate_limit": 8}
 
     def __init__(self, path: Optional[pathlib.Path] = None):
         if path is None:
@@ -217,8 +224,6 @@ class Config(PersistentDict):
 
         super().__init__(path, compress=False, **self.DEFAULTS)
 
-    def __getitem__(self, key):
-        if key not in self:
-            super().__setitem__(key, os.environ[f"TANKOBON_{key.upper()}"])
 
-        return super().__getitem__(key)
+# Only one instance should be initialised (this one).
+CONFIG = Config()
